==> toycars-0.3.0.tar.gz: toycars-0.3.0/src/Scoobie/ScParticles.h <==
/*
	Copyright (C) 2005 by Ruben Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  ScParticles.h
 *  Scoobie
 *
 *  Created by Ruben Zilibowitz on 13/01/05.
 *
 *  Based on code from SpriteWorld (SWParticles.c)
 *  See www.spriteworld.org
 *
 */

#ifndef SCPARTICLES_H
#define SCPARTICLES_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "ScComponent.h"
#include "ScTypes.h"

// ----------------------------------------------------------------
// Particle
// ----------------------------------------------------------------

const ScFloat kParticleDragConstant = 0.01;

struct Particle
{
	ScFloat	loc_x, loc_y;	// current location
	ScFloat	life;			// life remaining in seconds
	ScFloat	vec_x, vec_y;	// direction of movement of particle (should be normalised)
	ScFloat	speed;			// speed of travel (scalar value)
	
	// constructor method
	Particle(ScFloat inLoc_x = 0, ScFloat inLoc_y = 0, ScFloat inLife = -1, ScFloat inVec_x = 0, ScFloat inVec_y = 0, ScFloat inSpeed = 0)
		: loc_x(inLoc_x), loc_y(inLoc_y), life(inLife), vec_x(inVec_x), vec_y(inVec_y), speed(inSpeed)
	{}
	
	// step method
	Particle& step(ScFloat dt)
	{
		// calculate new position
		loc_x += dt * vec_x * speed;
		loc_y += dt * vec_y * speed;
		
		// apply drag
		if (speed > 0)
			speed -= dt * kParticleDragConstant * speed * speed;
		else
			speed += dt * kParticleDragConstant * speed * speed;
		
		// return reference to self so method can be used in an expression
		return *this;
	}
};

// ----------------------------------------------------------------
// ScParticles
// ----------------------------------------------------------------

class ScParticles : public ScComponent
{
public:
			ScParticles(long inMaxParticles, float inRed = 0, float inGreen = 1, float inBlue = 1);
			~ScParticles();
	
	void	draw();
	void	reset();
	void	newParticle(const Particle &p);
	
protected:
	void	process(ScFloat dt);
	
	const long	maxParticles;
	Particle*	particleArray;
	
	long	lowestUnusedElement;
	long	lastElementInUse;
	long	numberOfParticlesActive;
	
	float	red, green, blue;
};

#endif

==> toycars-0.3.3.tar.gz: toycars-0.3.3/src/Scoobie/ScParticles.h <==
/*
	Copyright (C) 2005 by Ruben Henner Zilibowitz <rzilibowitz@users.sourceforge.net>
	Part of the Toy Cars Project http://toycars.sourceforge.net

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License.
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY.

	See the COPYING file for more details.
*/

/*
 *  ScParticles.h
 *  Scoobie
 *
 *  Created by Ruben Henner Zilibowitz on 13/01/05.
 *
 *  Based on code from SpriteWorld (SWParticles.c)
 *  See www.spriteworld.org
 *
 */

#ifndef SCPARTICLES_H
#define SCPARTICLES_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "ScComponent.h"
#include "ScTypes.h"

// ----------------------------------------------------------------
// Particle
// ----------------------------------------------------------------

const ScFloat kParticleDragConstant = 0.01;

struct Particle
{
	ScFloat	loc_x, loc_y;	// current location
	ScFloat	life;			// life remaining in seconds
	ScFloat	vec_x, vec_y;	// direction of movement of particle (should be normalised)
	ScFloat	speed;			// speed of travel (scalar value)
	
	// constructor method
	Particle(ScFloat inLoc_x = 0, ScFloat inLoc_y = 0, ScFloat inLife = -1, ScFloat inVec_x = 0, ScFloat inVec_y = 0, ScFloat inSpeed = 0)
		: loc_x(inLoc_x), loc_y(inLoc_y), life(inLife), vec_x(inVec_x), vec_y(inVec_y), speed(inSpeed)
	{}
	
	// step method
	Particle& step(ScFloat dt)
	{
		// calculate new position
		loc_x += dt * vec_x * speed;
		loc_y += dt * vec_y * speed;
		
		// apply drag
		if (speed > 0)
			speed -= dt * kParticleDragConstant * speed * speed;
		else
			speed += dt * kParticleDragConstant * speed * speed;
		
		// return reference to self so method can be used in an expression
		return *this;
	}
};

// ----------------------------------------------------------------
// ScParticles
// ----------------------------------------------------------------

class ScParticles : public ScComponent
{
public:
			ScParticles(long inMaxParticles, float inRed = 0, float inGreen = 1, float inBlue = 1);
			~ScParticles();
	
	void	draw();
	void	reset();
	void	newParticle(const Particle &p);
	
protected:
	void	process(ScFloat dt);
	
	const long	maxParticles;
	Particle*	particleArray;
	
	long	lowestUnusedElement;
	long	lastElementInUse;
	long	numberOfParticlesActive;
	
	float	red, green, blue;
};

#endif
